==> toxiclibs-clj-0.2.0-20120309.162952-1.jar: toxi/geom/triangle2d.clj <==
(ns toxi.geom.triangle2d
  (:require
    [toxi.geom.utils :as utils]
    [toxi.math.core :as math])
  (:use
    [toxi.geom.protocols]
    [toxi.geom.core :only [vec2d vec3d line2d rect circle polygon2d triangle2d]]
    [toxi.geom.vec2d])
  (:import
    [toxi.geom.types Triangle2D]))

(defn equilateral-triangle2d
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.createEquilateralFrom"}
  ([line] (equilateral-triangle2d (:a line) (:b line)))
  ([a b]
    (let[m (interpolate a b 0.5)
         dir (sub a b)
         n (perpendicular dir)
         c (add m (normalize n (* (mag dir) (/ math/SQRT3 2))))]
      (triangle2d a b c)))
  ([x1 y1 x2 y2] (equilateral-triangle2d (vec2d x1 y1) (vec2d x2 y2))))

(defn- tri2d-from-barycentric
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.fromBarycentric"}
  [tri bary]
  (let [{:keys [x y z]} bary
        a (:a tri) b (:b tri) c (:c tri)
        px (+ (* (:x a) x) (* (:x b) y) (* (:x c) z))
        py (+ (* (:y a) x) (* (:y b) y) (* (:y c) z))]
    (vec2d px py)))

(defn- tri2d-clockwise?
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.isClockwise"}
  ([tri] (tri2d-clockwise? (:a tri) (:b tri) (:c tri)))
  ([a b c]
    (let [ax (:x a) ay (:y a)
          bx (:x b) by (:y b)
          cx (:x c) cy (:y c)]
      (pos? (- (* (- bx ax) (- cy ay)) (* (- cx ax) (- by ay)))))))

(defn- tri2d-flip
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.flipVertexOrder"}
  [tri]
  (assoc tri :b (:c tri) :c (:b tri)))

(defn- tri2d-centroid
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.computeCentroid"}
  [tri]
  (scale-n (add (:a tri) (:b tri) (:c tri)) 1/3))

(defn- tri2d-contains-point?
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.containsPoint"}
  [tri p]
  (if (or (= p (:a tri)) (= p (:b tri)) (= p (:c tri)))
    true
    (let [n1 (normalize (sub p (:a tri)))
          n2 (normalize (sub p (:b tri)))
          n3 (normalize (sub p (:c tri)))
          total (+ (Math/acos (dot n1 n2))
                  (Math/acos (dot n2 n3))
                  (Math/acos (dot n3 n1)))
          delta (math/abs (- total math/TWO_PI))]
      (<= delta 0.001))))

(defn- tri2d-area
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.getArea"}
  [tri]
  (* 0.5 (cross (sub (:b tri) (:a tri)) (sub (:c tri) (:a tri)))))

(defn- tri2d-circumference
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.getCircumference"}
  [tri]
  (+ (distance (:a tri) (:b tri))
    (distance (:b tri) (:c tri))
    (distance (:c tri) (:a tri))))

(defn- tri2d-adjust-size
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.adjustTriangleSizeBy"}
  ([tri offset] (adjust-size tri offset offset offset))
  ([tri offab offbc offca]
    (let [centroid (centroid tri)
          ab (offset-and-grow-by (line2d (:a tri) (:b tri)) offab 100000000 centroid)
          bc (offset-and-grow-by (line2d (:b tri) (:c tri)) offab 100000000 centroid)
          ca (offset-and-grow-by (line2d (:c tri) (:a tri)) offab 100000000 centroid)]
      {:a (:pos (intersect-lines ab ca))
       :b (:pos (intersect-lines ab bc))
       :c (:pos (intersect-lines bc ca))})))

(defn- tri2d-circum-circle
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.getCircumCircle"}
  [tri]
  (let [a (:a tri) b (:b tri) c (:c tri)
        cr (cross (bisect a b) (bisect b c))]
    (if (>= (math/abs (:z cr)) math/EPS)
        (let [orig (vec2d (/ (:x cr) (:z cr)) (/ (:y cr) (:z cr)))
              sa (distance a b)
              sb (distance b c)
              sc (distance c a)
              denom (* (+ sa sb sc) (- (+ sb sc) sa) (+ (- sa sb) sc) (- (+ sa sb) sc))
              radius (/ (* sa sb sc) (Math/sqrt denom))]
          (circle orig radius)))))

(defn- tri2d-closest-point-to
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.getClosestPointTo"}
  [tri p]
  (let [a (:a tri) b (:b tri) c (:c tri)
        rab (line-closest-point-to (line2d a b) p)
        rbc (line-closest-point-to (line2d b c) p)
        rca (line-closest-point-to (line2d c a) p)
        dab (mag-squared (sub p rab))
        dbc (mag-squared (sub p rbc))
        dca (mag-squared (sub p rca))
        mind (min dab dbc dca)]
    (cond
      (= mind dab) rab
      (= mind dbc) rbc
      :else rca)))

(defn- tri2d->polygon2d
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.toPolygon2D"}
  [tri]
  (polygon2d [(:a tri) (:b tri) (:c tri)]))

(defn- tri2d-bounds
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.getBounds"}
  [tri]
  (let [a (:a tri) b (:b tri) c (:c tri)
        minx (min (:x a) (:x b) (:x c))
        maxx (max (:x a) (:x b) (:x c))
        miny (min (:y a) (:y b) (:y c))
        maxy (max (:y a) (:y b) (:y c))]
    (rect minx miny (- maxx minx) (- maxy miny))))

(defn- tri2d-random-point
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.getRandomPoint"}
  [tri]
  (let [b1 (math/random)
        b2 (math/random (- 1.0 b1))
        b3 (- 1.0 (+ b1 b2))]
    (from-barycentric tri (vec3d (shuffle [b1 b2 b3])))))

(defn- tri2d-edges
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.getEdges"}
  [tri]
  (let [a (:a tri) b (:b tri) c (:c tri)]
    [(line2d a b) (line2d b c) (line2d c a)]))

(defn- tri2d-intersect-triangles
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.intersectsTriangle"}
  [tri tri2]
  (let [a (:a tri) b (:b tri) c (:c tri)
        a2 (:a tri2) b2 (:b tri2) c2 (:c tri2)]
    (if (or
          (tri2d-contains-point? tri a2)
          (tri2d-contains-point? tri b2)
          (tri2d-contains-point? tri c2)
          (tri2d-contains-point? tri2 a)
          (tri2d-contains-point? tri2 b)
          (tri2d-contains-point? tri2 c))
      true
      (let [ea (edges tri) eb (edges tri2)
            res (loop [la ea]
                  (if-not (nil? (seq la))
                    (if-not
                      (nil?
                        (loop [lb eb]
                          (if-not (nil? (seq lb))
                            (let [type (:type (intersect-lines (first la) (first lb)))]
                              (if (or (= type :intersect) (= type :coincident))
                                true
                                (recur (rest lb)))))))
                      true (recur (rest la)))))]
        (true? res)))))

(defn extend-triangle2d
  [type]
  (extend type
    IShape {
      :bounds tri2d-bounds
      :centroid tri2d-centroid
      :contains-point? tri2d-contains-point?
      :random-point tri2d-random-point
    }
    IShape2D {
      :area tri2d-area
      :bounding-circle tri2d-circum-circle
      :circumference tri2d-circumference
      :edges tri2d-edges
      :->polygon2d tri2d->polygon2d
    }
    ITriangle {
      :adjust-size tri2d-adjust-size
		  :clockwise? tri2d-clockwise?
		  :from-barycentric tri2d-from-barycentric
		  :intersect-triangles tri2d-intersect-triangles
		  :triangle-closest-point-to tri2d-closest-point-to
    }
  ))

(extend-triangle2d Triangle2D)

==> toxiclibs-clj-0.2.0-20120314.000135-3.jar: toxi/geom/triangle2d.clj <==
(ns toxi.geom.triangle2d
  (:require
    [toxi.geom.utils :as utils]
    [toxi.math.core :as math])
  (:use
    [toxi.geom.core]
    [toxi.geom.vec2d])
  (:import
    [toxi.geom.types Triangle2D]))

(defn equilateral-triangle2d
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.createEquilateralFrom"}
  ([line] (equilateral-triangle2d (:a line) (:b line)))
  ([a b]
    (let[m (interpolate a b 0.5)
         dir (sub a b)
         n (perpendicular dir)
         c (add m (normalize n (* (mag dir) (/ math/SQRT3 2))))]
      (triangle2d a b c)))
  ([x1 y1 x2 y2] (equilateral-triangle2d (vec2d x1 y1) (vec2d x2 y2))))

(defn- tri2d-from-barycentric
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.fromBarycentric"}
  [tri bary]
  (let [{:keys [x y z]} bary
        a (:a tri) b (:b tri) c (:c tri)
        px (+ (* (:x a) x) (* (:x b) y) (* (:x c) z))
        py (+ (* (:y a) x) (* (:y b) y) (* (:y c) z))]
    (vec2d px py)))

(defn- tri2d-clockwise?
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.isClockwise"}
  ([tri] (tri2d-clockwise? (:a tri) (:b tri) (:c tri)))
  ([a b c]
    (let [ax (:x a) ay (:y a)
          bx (:x b) by (:y b)
          cx (:x c) cy (:y c)]
      (pos? (- (* (- bx ax) (- cy ay)) (* (- cx ax) (- by ay)))))))

(defn- tri2d-flip
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.flipVertexOrder"}
  [tri]
  (assoc tri :b (:c tri) :c (:b tri)))

(defn- tri2d-centroid
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.computeCentroid"}
  [tri]
  (scale-n (add (:a tri) (:b tri) (:c tri)) 1/3))

(defn- tri2d-contains-point?
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.containsPoint"}
  [tri p]
  (if (or (= p (:a tri)) (= p (:b tri)) (= p (:c tri)))
    true
    (let [n1 (normalize (sub p (:a tri)))
          n2 (normalize (sub p (:b tri)))
          n3 (normalize (sub p (:c tri)))
          total (+ (Math/acos (dot n1 n2))
                  (Math/acos (dot n2 n3))
                  (Math/acos (dot n3 n1)))
          delta (math/abs (- total math/TWO_PI))]
      (<= delta 0.001))))

(defn- tri2d-area
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.getArea"}
  [tri]
  (* 0.5 (cross (sub (:b tri) (:a tri)) (sub (:c tri) (:a tri)))))

(defn- tri2d-circumference
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.getCircumference"}
  [tri]
  (+ (distance (:a tri) (:b tri))
    (distance (:b tri) (:c tri))
    (distance (:c tri) (:a tri))))

(defn- tri2d-adjust-size
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.adjustTriangleSizeBy"}
  ([tri offset] (adjust-size tri offset offset offset))
  ([tri offab offbc offca]
    (let [centroid (centroid tri)
          ab (offset-and-grow-by (line2d (:a tri) (:b tri)) offab 100000000 centroid)
          bc (offset-and-grow-by (line2d (:b tri) (:c tri)) offab 100000000 centroid)
          ca (offset-and-grow-by (line2d (:c tri) (:a tri)) offab 100000000 centroid)]
      {:a (:pos (intersect ab ca))
       :b (:pos (intersect ab bc))
       :c (:pos (intersect bc ca))})))

(defn- tri2d-circum-circle
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.getCircumCircle"}
  [tri]
  (let [a (:a tri) b (:b tri) c (:c tri)
        cr (cross (bisect a b) (bisect b c))]
    (if (>= (math/abs (:z cr)) math/EPS)
        (let [orig (vec2d (/ (:x cr) (:z cr)) (/ (:y cr) (:z cr)))
              sa (distance a b)
              sb (distance b c)
              sc (distance c a)
              denom (* (+ sa sb sc) (- (+ sb sc) sa) (+ (- sa sb) sc) (- (+ sa sb) sc))
              radius (/ (* sa sb sc) (Math/sqrt denom))]
          (circle orig radius)))))

(defn- tri2d-closest-point-to
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.getClosestPointTo"}
  [tri p]
  (let [a (:a tri) b (:b tri) c (:c tri)
        rab (closest-point (line2d a b) p)
        rbc (closest-point (line2d b c) p)
        rca (closest-point (line2d c a) p)
        dab (mag-squared (sub p rab))
        dbc (mag-squared (sub p rbc))
        dca (mag-squared (sub p rca))
        mind (min dab dbc dca)]
    (cond
      (= mind dab) rab
      (= mind dbc) rbc
      :else rca)))

(defn- tri2d->polygon2d
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.toPolygon2D"}
  [tri]
  (polygon2d [(:a tri) (:b tri) (:c tri)]))

(defn- tri2d-bounds
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.getBounds"}
  [tri]
  (let [a (:a tri) b (:b tri) c (:c tri)
        minx (min (:x a) (:x b) (:x c))
        maxx (max (:x a) (:x b) (:x c))
        miny (min (:y a) (:y b) (:y c))
        maxy (max (:y a) (:y b) (:y c))]
    (rect minx miny (- maxx minx) (- maxy miny))))

(defn- tri2d-random-point
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.getRandomPoint"}
  [tri]
  (let [b1 (math/random)
        b2 (math/random (- 1.0 b1))
        b3 (- 1.0 (+ b1 b2))]
    (from-barycentric tri (vec3d (shuffle [b1 b2 b3])))))

(defn- tri2d-edges
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.getEdges"}
  [tri]
  (let [a (:a tri) b (:b tri) c (:c tri)]
    [(line2d a b) (line2d b c) (line2d c a)]))

(defn- tri2d-intersect-triangles
  {:added "0.1"
   :java-id "toxi.geom.Triangle2D.intersectsTriangle"}
  [tri tri2]
  (let [a (:a tri) b (:b tri) c (:c tri)
        a2 (:a tri2) b2 (:b tri2) c2 (:c tri2)]
    (if (or
          (tri2d-contains-point? tri a2)
          (tri2d-contains-point? tri b2)
          (tri2d-contains-point? tri c2)
          (tri2d-contains-point? tri2 a)
          (tri2d-contains-point? tri2 b)
          (tri2d-contains-point? tri2 c))
      true
      (let [ea (edges tri) eb (edges tri2)
            res (loop [la ea]
                  (if-not (nil? (seq la))
                    (if-not
                      (nil?
                        (loop [lb eb]
                          (if-not (nil? (seq lb))
                            (let [type (:type (intersect (first la) (first lb)))]
                              (if (or (= type :intersect) (= type :coincident))
                                true
                                (recur (rest lb)))))))
                      true (recur (rest la)))))]
        (true? res)))))

(defn extend-triangle2d
  [type]
  (extend type
    IIntersectable {
		  :intersect tri2d-intersect-triangles
    }
    IProximity {
		  :closest-point tri2d-closest-point-to
    }
    IShape {
      :bounds tri2d-bounds
      :centroid tri2d-centroid
      :contains-point? tri2d-contains-point?
      :random-point tri2d-random-point
    }
    IShape2D {
      :area tri2d-area
      :bounding-circle tri2d-circum-circle
      :circumference tri2d-circumference
      :edges tri2d-edges
      :->polygon2d tri2d->polygon2d
    }
    ITriangle {
      :adjust-size tri2d-adjust-size
		  :clockwise? tri2d-clockwise?
		  :from-barycentric tri2d-from-barycentric
    }
  ))

(extend-triangle2d Triangle2D)
